package com.tiffintom.partner1.fragments;

import static com.tiffintom.partner1.utils.Constants.TAPTOPAY;

import android.Manifest;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothManager;
import android.content.BroadcastReceiver;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.DialogFragment;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.google.android.material.button.MaterialButton;
import com.google.gson.Gson;
import com.stripe.stripeterminal.Terminal;
import com.stripe.stripeterminal.TerminalApplicationDelegate;
import com.stripe.stripeterminal.external.callable.BluetoothReaderListener;
import com.stripe.stripeterminal.external.callable.Callback;
import com.stripe.stripeterminal.external.callable.Cancelable;
import com.stripe.stripeterminal.external.callable.DiscoveryListener;
import com.stripe.stripeterminal.external.callable.PaymentIntentCallback;
import com.stripe.stripeterminal.external.callable.ReaderCallback;
import com.stripe.stripeterminal.external.callable.TerminalListener;
import com.stripe.stripeterminal.external.callable.UsbReaderListener;
import com.stripe.stripeterminal.external.models.BatteryStatus;
import com.stripe.stripeterminal.external.models.CollectConfiguration;
import com.stripe.stripeterminal.external.models.ConnectionConfiguration;
import com.stripe.stripeterminal.external.models.ConnectionStatus;
import com.stripe.stripeterminal.external.models.DiscoveryConfiguration;
import com.stripe.stripeterminal.external.models.DiscoveryMethod;
import com.stripe.stripeterminal.external.models.PaymentIntent;
import com.stripe.stripeterminal.external.models.PaymentStatus;
import com.stripe.stripeterminal.external.models.Reader;
import com.stripe.stripeterminal.external.models.ReaderDisplayMessage;
import com.stripe.stripeterminal.external.models.ReaderEvent;
import com.stripe.stripeterminal.external.models.ReaderInputOptions;
import com.stripe.stripeterminal.external.models.ReaderSoftwareUpdate;
import com.stripe.stripeterminal.external.models.SimulateReaderUpdate;
import com.stripe.stripeterminal.external.models.SimulatedCard;
import com.stripe.stripeterminal.external.models.SimulatorConfiguration;
import com.stripe.stripeterminal.external.models.TerminalException;
import com.stripe.stripeterminal.log.LogLevel;
import com.tiffintom.partner1.MyApp;
import com.tiffintom.partner1.R;
import com.tiffintom.partner1.adapters.CardReaderAdapter;
import com.tiffintom.partner1.common.CustomTerminalEventListener;
import com.tiffintom.partner1.common.CustomTokenProvider;
import com.tiffintom.partner1.common.Validators;
import com.tiffintom.partner1.interfaces.DialogDismissDataListener;
import com.tiffintom.partner1.models.ApiError;
import com.tiffintom.partner1.models.MerchantBusinesses;
import com.tiffintom.partner1.models.MerchantCardReader;
import com.tiffintom.partner1.network.ApiEndPoints;
import com.tiffintom.partner1.utils.Constants;
import com.tiffintom.partner1.utils.LogUtils;

import org.jetbrains.annotations.NotNull;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Objects;

/**
 * Created by Amrish on 11-11-2020.
 */
public class StripeBluetoothCardReaderPaymentFragment extends DialogFragment implements BluetoothReaderListener, UsbReaderListener, TerminalListener {
    @Override
    public int getTheme() {
        return R.style.MyDialog;
    }

    @Override
    public void onStart() {
        super.onStart();
        getDialog().getWindow().setLayout(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT);
    }

    private static final int RE_DISCOVER = 101;
    private TextView tvMessage;
    private ProgressBar pbLoading;
    private Float amount;
    private MerchantCardReader cardReader;
    private DialogDismissDataListener dialogDismissListener;
    private Cancelable cancelableDiscovery;
    private MerchantBusinesses merchantBusinesses = MyApp.getInstance().getMyPreferences().getMerchantBusinesses();
    boolean readersFound = false, readerSelectionDisplayed = false;
    private MaterialButton btnCancel, btnConfirm, btnReStart;
    private ArrayList<Object> cardReaderObjects = new ArrayList<>();
    private LinearLayout llReaderView, llLoaderView;
    private RecyclerView rvReaders;
    private CardReaderAdapter cardReaderAdapter;
    private boolean isTapToPay,isTapToPayReaderDisconncetedMannuly = false;
    private MyApp myApp;

    public static StripeBluetoothCardReaderPaymentFragment getInstance(MerchantCardReader cardReader, float amount, boolean isTapToPay) {
        StripeBluetoothCardReaderPaymentFragment rejectReasonDialog = new StripeBluetoothCardReaderPaymentFragment();
        Bundle bundle = new Bundle();
        bundle.putString("card_reader", new Gson().toJson(cardReader));
        bundle.putFloat("amount", amount);
        bundle.putBoolean(TAPTOPAY, isTapToPay);
        rejectReasonDialog.setArguments(bundle);
        return rejectReasonDialog;
    }

    public void setDialogDismissListener(DialogDismissDataListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        return inflater.inflate(R.layout.dialog_stripe_bluetooth_fragment, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        TerminalApplicationDelegate.onCreate(getActivity().getApplication());
        LocalBroadcastManager.getInstance(getActivity()).registerReceiver(broadcastReceiver, new IntentFilter(Constants.CARD_READER_MESSAGE));
        if (getArguments() != null) {
            cardReader = new Gson().fromJson(getArguments().getString("card_reader"), MerchantCardReader.class);
            amount = getArguments().getFloat("amount");
            isTapToPay = getArguments().getBoolean(TAPTOPAY, false);
        }
        initViews(view);
        setListeners();
        //  requestBluetoothPermissions();
        askForPermission();
    }

    private void initViews(View view) {
        myApp = MyApp.getInstance();
        pbLoading = view.findViewById(R.id.pbLoading);
        tvMessage = view.findViewById(R.id.tvMessage);
        btnCancel = view.findViewById(R.id.btnCancel);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnReStart = view.findViewById(R.id.btnReStart);
        llReaderView = view.findViewById(R.id.llReadersView);
        llLoaderView = view.findViewById(R.id.llLoaderView);
        llReaderView.setVisibility(View.GONE);
        rvReaders = view.findViewById(R.id.rvCardReaders);
        rvReaders.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        cardReaderAdapter = new CardReaderAdapter(cardReaderObjects, (position, data) -> {
            if (data instanceof Reader) {
                llLoaderView.setVisibility(View.VISIBLE);
                llReaderView.setVisibility(View.GONE);
                connectReader((Reader) data, cardReader.s_location_id);
            }
        });
        rvReaders.setAdapter(cardReaderAdapter);
        btnCancel.setVisibility(View.GONE);
        enableBluetooth();
    }

    private void enableBluetooth() {

        try {
            if (ContextCompat.checkSelfPermission(getActivity(), Manifest.permission.BLUETOOTH_CONNECT) == PackageManager.PERMISSION_DENIED) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                    ActivityCompat.requestPermissions(getActivity(), new String[]{Manifest.permission.BLUETOOTH_CONNECT}, 2);
                    return;
                }
            }

            BluetoothAdapter bluetoothAdapter;
            BluetoothManager bluetoothManager = (BluetoothManager) getActivity().getSystemService(Context.BLUETOOTH_SERVICE);

            if (Build.VERSION.SDK_INT >= 31) {
                bluetoothAdapter = bluetoothManager.getAdapter();
            } else {
                bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
            }

            if (!bluetoothAdapter.isEnabled()) {
                bluetoothAdapter.enable();
            }
/*
        if (BluetoothAdapter.getDefaultAdapter() != null &&
                !BluetoothAdapter.getDefaultAdapter().isEnabled()) {
            BluetoothAdapter.getDefaultAdapter().enable();
        }
*/
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static void triggerRebirth(Context context) {
        PackageManager packageManager = context.getPackageManager();
        Intent intent = packageManager.getLaunchIntentForPackage(context.getPackageName());
        ComponentName componentName = intent.getComponent();
        Intent mainIntent = Intent.makeRestartActivityTask(componentName);
        context.startActivity(mainIntent);
        Runtime.getRuntime().exit(0);
    }

    private void setListeners() {
        btnReStart.setOnClickListener(v -> {
            MyApp.getInstance().getMyPreferences().saveReStartFromCardReader(true);
            triggerRebirth(requireContext());
        });
        btnCancel.setOnClickListener(v -> {
            //turnOnOffDeviceBluetooth();
            dismissAllowingStateLoss();
        });
        tvMessage.setOnClickListener(v -> {
            ClipboardManager clipboard = (ClipboardManager) getActivity().getSystemService(Context.CLIPBOARD_SERVICE);
            ClipData clip = ClipData.newPlainText("label", tvMessage.getText().toString());
            clipboard.setPrimaryClip(clip);
        });

    }

    private void turnOnOffDeviceBluetooth() {
        try {
            BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
            if (bluetoothAdapter.isEnabled()) {
                bluetoothAdapter.disable();
                // ToastUtils.makeToast(getActivity(), "BLUETOOTH TURNED OFF.");
            }
            new Handler(Objects.requireNonNull(Looper.myLooper()))
                    .postDelayed(() -> {
                        if (!bluetoothAdapter.isEnabled()) {
                            bluetoothAdapter.enable();
                            //   ToastUtils.makeToast(getActivity(), "BLUETOOTH TURNED ON.");
                        }
                    }, 1000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void initialize() {
        try {
            if(myApp.cancelableDiscovery!=null){
                myApp.cancelableDiscovery.cancel(callback);
            }
            if (!Terminal.isInitialized()) {
                Terminal.initTerminal(getActivity().getApplicationContext(), LogLevel.VERBOSE, new CustomTokenProvider(cardReader.s_location_id, merchantBusinesses.id), new CustomTerminalEventListener(getActivity()));
            } else {
                LogUtils.e("Terminal already initialized");
            }
        } catch (TerminalException e) {
            LogUtils.e("Terminal Error::" + e.getErrorMessage());
            if (dialogDismissListener != null)
                dialogDismissListener.onDialogDismiss("Location services are required in order to initialize the Terminal", "", "", false, false,"");
            dismissAllowingStateLoss();
        }
        Terminal.getInstance().setTerminalListener(this);
        Reader connectedReader = Terminal.getInstance().getConnectedReader();
        isTapToPayReaderDisconncetedMannuly = false;
        if (connectedReader != null) {
            if (myApp.lastConnectedReader.equalsIgnoreCase(cardReader.connectivity)) {
                requireActivity().runOnUiThread(() -> {
                    stripeReader = connectedReader;
                    cardReader.s_terminal_id = connectedReader.getId();
                    createPaymentIntent();
                });
            } else {
                isTapToPayReaderDisconncetedMannuly = true;
                Terminal.getInstance().disconnectReader(disConnectCallback);
                myApp.lastConnectedReader = "";
                discoverReaders();
            }

        } else {
            discoverReaders();
        }
    }


    private void askForPermission() {
        try {

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                final int scanPermission = ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.BLUETOOTH_SCAN);
                final int connectPermission = ActivityCompat.checkSelfPermission(getActivity(), Manifest.permission.BLUETOOTH_CONNECT);
                if (scanPermission == PackageManager.PERMISSION_GRANTED && connectPermission == PackageManager.PERMISSION_GRANTED) {
                    initialize();
                } else {
                    // If we don't have them yet, request them before doing anything else
                    final String[] permissions = {Manifest.permission.BLUETOOTH_CONNECT, Manifest.permission.BLUETOOTH_SCAN};
                    requestPermissions(permissions, 100);
                }
            } else {
                initialize();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //Simulated
    boolean isSimulated = false;

    private void discoverReaders() {
        try {
            LogUtils.e("Discovering Readers");
            //Charges declined 4000000000000002
            //Success Discover Type 6011111111111117
            if (isSimulated) {
                SimulatorConfiguration simulatorConfiguration = new SimulatorConfiguration(SimulateReaderUpdate.RANDOM, new SimulatedCard("4242424242424242"), 0L);
                Terminal.getInstance().setSimulatorConfiguration(simulatorConfiguration);
            }
            sendMessage("Discovering readers...", 0);
            DiscoveryConfiguration discoveryConfig;
            //if (cardReader != null && cardReader.isUsb != null && cardReader.isUsb) {
            if (isTapToPay) {
                discoveryConfig = new DiscoveryConfiguration(30, DiscoveryMethod.LOCAL_MOBILE, isSimulated, cardReader.s_location_id);
            } else {
                if (MyApp.getInstance().getMyPreferences().getIsUbEnableStatus()) {
                    discoveryConfig = new DiscoveryConfiguration(30, DiscoveryMethod.USB, isSimulated);
                } else {
                    discoveryConfig = new DiscoveryConfiguration(30, DiscoveryMethod.BLUETOOTH_SCAN, isSimulated);
                }
            }
            cancelableDiscovery = Terminal.getInstance().discoverReaders(discoveryConfig, new DiscoveryListener() {
                @Override
                public void onUpdateDiscoveredReaders(@NonNull List<Reader> list) {
                    LogUtils.e("TERMINAL:::Discovery result::" + list.size());
                    /*getActivity().runOnUiThread(() -> {
                        ArrayList<Reader> readerArrayList = new ArrayList<>(list);
                        llLoaderView.setVisibility(View.GONE);
                        llReaderView.setVisibility(View.VISIBLE);
                        ArrayList<Object> localObjects = new ArrayList<>(list);
                        cardReaderAdapter.updateReaders(localObjects);
                    });*/
                    getActivity().runOnUiThread(() -> {
                        // if size is 1 connect reader by default instead of display only 1 list in dialog
                        //ToastUtils.makeLongToast(getActivity(),"size of the list is "+list.size());
                        if (list.size() == 1) {
                            llLoaderView.setVisibility(View.VISIBLE);
                            llReaderView.setVisibility(View.GONE);
                            connectReader(list.get(0), cardReader.s_location_id);
                        } else {
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                                List<Reader> filterSerialList = new ArrayList<>();
                                // store setting filter name in list
                                for (int i = 0; i < list.size(); i++) {
                                    LogUtils.e("List:::Serial numner::" + (list.get(i).getSerialNumber()) + " card reader serial number " + cardReader.serial_number);
                                    if (cardReader.serial_number != null && list.get(i).getSerialNumber() != null && Objects.equals(list.get(i).getSerialNumber().toLowerCase(Locale.getDefault()), cardReader.serial_number.toLowerCase(Locale.getDefault()))) {
                                        filterSerialList.add(list.get(i));
                                    }
                                }
                                if (!filterSerialList.isEmpty()) {
                                    llLoaderView.setVisibility(View.VISIBLE);
                                    llReaderView.setVisibility(View.GONE);
                                    connectReader(filterSerialList.get(0), cardReader.s_location_id);
                                } else {
                                    llLoaderView.setVisibility(View.GONE);
                                    llReaderView.setVisibility(View.VISIBLE);
                                    ArrayList<Object> localObjects = new ArrayList<>(list);
                                    cardReaderAdapter.updateReaders(localObjects);
                                }
                            }
                        }
                    });
                }
            }, cancelDiscoveryCallback);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    final Callback disConnectCallback = new Callback() {
        @Override
        public void onSuccess() {
            LogUtils.e("On success");
            myApp.lastConnectedReader = "";
        }

        @Override
        public void onFailure(@NotNull TerminalException e) {
            LogUtils.e("on failure: " + e.getErrorMessage());
        }
    };

    final Callback callback = new Callback() {
        @Override
        public void onSuccess() {
            LogUtils.e("On success");
        }

        @Override
        public void onFailure(@NotNull TerminalException e) {
            LogUtils.e("on failure: " + e.getErrorMessage());
        }
    };

    final Callback cancelDiscoveryCallback = new Callback() {
        @Override
        public void onSuccess() {
            LogUtils.e("On success");
        }

        @Override
        public void onFailure(@NotNull TerminalException e) {
            if (e.getErrorCode() == TerminalException.TerminalErrorCode.CANCELED) {
                LogUtils.e("on CANCELED: " + e.getErrorMessage());
            }else{
                if (e.getErrorCode() == TerminalException.TerminalErrorCode.BLUETOOTH_SCAN_TIMED_OUT) {
                    sendMessage("Discovering readers taking more time than expected, Please Retry.", RE_DISCOVER);
                }   else{
                    sendMessage(""+e.getErrorMessage(), 2);
                }
            }
            LogUtils.e("on failure: " + e.getErrorMessage());
        }
    };

    Reader stripeReader;

    private void connectReader(Reader reader, String locationId) {
        sendMessage("Connecting to terminal: " + reader.getSerialNumber(), 0);
        //if (cardReader != null && cardReader.isUsb != null && cardReader.isUsb) {
        if (isTapToPay) {
            ConnectionConfiguration.LocalMobileConnectionConfiguration connectionConfig =
                    new ConnectionConfiguration.LocalMobileConnectionConfiguration(locationId);
            Terminal.getInstance().connectLocalMobileReader(reader, connectionConfig, new ReaderCallback() {
                @Override
                public void onSuccess(Reader r) {
                    myApp.lastConnectedReader = cardReader.connectivity;
                    getActivity().runOnUiThread(() -> {
                        stripeReader = r;
                        cardReader.s_terminal_id = r.getId();
                        createPaymentIntent();

                    });

                }

                @Override
                public void onFailure(TerminalException e) {
                    e.printStackTrace();
                    sendMessage("Terminal Exception\nCode:" + e.getErrorCode().toString() + "\n" + e.getErrorMessage() + "\n", 2);

                }
            });
        } else {
            if (MyApp.getInstance().getMyPreferences().getIsUbEnableStatus()) {
                Terminal.getInstance().connectUsbReader(reader, new ConnectionConfiguration.UsbConnectionConfiguration(locationId), this, new ReaderCallback() {
                    @Override
                    public void onSuccess(Reader r) {
                        myApp.lastConnectedReader = cardReader.connectivity;
                        getActivity().runOnUiThread(() -> {
                            stripeReader = r;
                            cardReader.s_terminal_id = r.getId();
                            createPaymentIntent();

                        });

                    }

                    @Override
                    public void onFailure(TerminalException e) {
                        e.printStackTrace();
                        sendMessage("Terminal Exception\nCode:" + e.getErrorCode().toString() + "\n" + e.getErrorMessage() + "\n", 2);

                    }
                });
            } else {
                ConnectionConfiguration.BluetoothConnectionConfiguration connectionConfig =
                        new ConnectionConfiguration.BluetoothConnectionConfiguration(locationId);
                Terminal.getInstance().connectBluetoothReader(reader, connectionConfig, this, new ReaderCallback() {
                    @Override
                    public void onSuccess(Reader r) {
                        myApp.lastConnectedReader = cardReader.connectivity;
                        myApp.getMyPreferences().saveCardReaderConnect( cardReader.connectivity);
                        myApp.getMyPreferences().saveSerialNumber(r.getSerialNumber());
                        myApp.getMyPreferences().saveLocationId(locationId);
                        getActivity().runOnUiThread(() -> {
                            stripeReader = r;
                            cardReader.s_terminal_id = r.getId();
                            createPaymentIntent();

                        });

                    }

                    @Override
                    public void onFailure(TerminalException e) {
                        e.printStackTrace();
                        sendMessage("Terminal Exception\nCode:" + e.getErrorCode().toString() + "\n" + e.getErrorMessage() + "\n", 2);

                    }
                });
            }
        }
    }

    private HashMap<String, String> generatePaymentIntentParams() {
        HashMap<String, String> params = new HashMap<>();

        params.put("amount", MyApp.df.format(amount * 100));
        params.put("currency", "GBP");
        params.put("transaction_type", merchantBusinesses.s_account_id == null ? "merchant" : "connect");
        params.put("business_id", merchantBusinesses.id);
        if (cardReader.s_terminal_id != null)
            params.put("s_terminal_id", cardReader.s_terminal_id);
        else params.put("s_terminal_id", cardReader.id);
        return params;
    }

    private void createPaymentIntent() {
        AndroidNetworking.post(ApiEndPoints.PAYMENT_INTENT)
                .addBodyParameter(generatePaymentIntentParams())
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {

                        try {
                            if (response.has("payment_intent")) {
                                JSONObject paymentIntent = response.getJSONObject("payment_intent");
                                String paymentSecret = paymentIntent.getString("client_secret");
                                Terminal.getInstance().retrievePaymentIntent(paymentSecret, new PaymentIntentCallback() {
                                    //  @OptIn(markerClass = OnReaderTips.class)//TODO change terminal version to 2.11
                                    @Override
                                    public void onSuccess(@NotNull PaymentIntent paymentIntent) {
                                        sendMessage("Please present card", 0);
                                        getActivity().runOnUiThread(() -> {
                                            collectPaymentMethod(paymentIntent);
                                        });
                                    }

                                    @Override
                                    public void onFailure(@NotNull TerminalException e) {
                                        LogUtils.e("Error while retriving intent");
                                        if (e.getErrorCode() == TerminalException.TerminalErrorCode.DECLINED_BY_STRIPE_API
                                                || e.getErrorCode() == TerminalException.TerminalErrorCode.CANCELED) {
                                            paymentDeclinedByStripe(e.getErrorCode());
                                        }
                                        e.printStackTrace();
                                        sendMessage("Payment Failed", 2);
                                    }
                                });

                            } else {
                                sendMessage("Something went wrong", 2);
                            }
                        } catch (JSONException e) {
                            e.printStackTrace();
                            sendMessage("Something went wrong " + e.getMessage(), 2);
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        anError.printStackTrace();
                        LogUtils.e("MOTO PAYMENT ERROR");
                        LogUtils.e(anError.getErrorBody());
                        if (anError.getErrorCode() > 400) {
                            ApiError apiError = anError.getErrorAsObject(ApiError.class);
                            sendMessage("Payment Failed: " + apiError.getMessage(), 2);
                        } else {
                            sendMessage("Payment Failed: ", 2);
                        }
                    }
                });
    }

    private void paymentDeclinedByStripe(TerminalException.TerminalErrorCode terminalErrorCode) {
        if (terminalErrorCode != null) {
            getActivity().runOnUiThread(() -> {
                if (dialogDismissListener != null)
                    dialogDismissListener.onDialogDismiss(null, cardName, cardNumber, terminalErrorCode == TerminalException.TerminalErrorCode.DECLINED_BY_STRIPE_API,
                            terminalErrorCode == TerminalException.TerminalErrorCode.CANCELED,"");
                dismissAllowingStateLoss();
            });
        }
    }

    private void collectPaymentMethod(PaymentIntent paymentIntent) {
        try {
            LogUtils.e("Collecting Payment method");
            CollectConfiguration collectConfiguration = new CollectConfiguration(false);
            Terminal.getInstance().collectPaymentMethod(paymentIntent,
                    new PaymentIntentCallback() {
                        //  @OptIn(markerClass = OnReaderTips.class)//TODO change terminal version to 2.11
                        @Override
                        public void onSuccess(PaymentIntent paymentIntent) {
                            LogUtils.e("Processing payment intent");
                            sendMessage("Processing payment", 0);
                            // Placeholder for processing paymentIntent
                            processPayment(paymentIntent);
                        }

                        @Override
                        public void onFailure(TerminalException exception) {
                            if (exception.getErrorCode() == TerminalException.TerminalErrorCode.DECLINED_BY_STRIPE_API
                                    || exception.getErrorCode() == TerminalException.TerminalErrorCode.CANCELED) {
                                paymentDeclinedByStripe(exception.getErrorCode());
                            }
                            // Placeholder for handling exception
                            sendMessage("Error while processing payment", 2);
                            LogUtils.e(new Gson().toJson(exception));
                            LogUtils.e("Error while collecting payment method:" + exception.getErrorCode());
                            exception.printStackTrace();
                        }
                    }, collectConfiguration);
        } catch (Exception e) {
            sendMessage("Something went wrong " + e.getMessage(), 2);
            e.printStackTrace();
        }
    }

    private void processPayment(PaymentIntent paymentIntent) {
        try {
            LogUtils.e("Process payment intent");
            Terminal.getInstance().processPayment(paymentIntent, new PaymentIntentCallback() {
                // @OptIn(markerClass = OnReaderTips.class)//TODO change terminal version to 2.11
                @Override
                public void onSuccess(@NotNull PaymentIntent paymentIntent) {
                    capturePaymentIntent(paymentIntent);
                }

                @Override
                public void onFailure(@NotNull TerminalException e) {
                    if (e.getErrorCode() == TerminalException.TerminalErrorCode.DECLINED_BY_STRIPE_API
                            || e.getErrorCode() == TerminalException.TerminalErrorCode.CANCELED) {
                        paymentDeclinedByStripe(e.getErrorCode());
                    }
                    LogUtils.e(e.getErrorMessage());
                    LogUtils.e("Process payment error");
                    sendMessage(e.getErrorMessage(), 2);
                }
            });
        } catch (Exception e) {
            sendMessage("Something went wrong " + e.getMessage(), 2);
            e.printStackTrace();
        }
    }

    private HashMap<String, String> getCapturePaymentIntentParams(String payment_intent_id) {
        HashMap<String, String> params = new HashMap<>();

        params.put("payment_intent_id", payment_intent_id);
        params.put("business_id", merchantBusinesses.id);
        if (cardReader.s_terminal_id != null)
            params.put("s_terminal_id", cardReader.s_terminal_id);
        else params.put("s_terminal_id", cardReader.id);
        return params;
    }

    String cardName = "";
    String cardNumber = "";

    private void capturePaymentIntent(final PaymentIntent paymentIntent) {
        try {
            AndroidNetworking.post(ApiEndPoints.CAPTURE_PAYMENT_INTENT)
                    .addBodyParameter(getCapturePaymentIntentParams(paymentIntent.getId()))
                    .build()
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            LogUtils.e("Response", " " + response.toString());
                            sendMessage("Payment successful", 1);
                            // Get card brand name and number manually , removing PaymentMethod class
                            String tipFromCardReader="";
                            try {
                                if (response != null) {
                                    cardName = "";
                                    cardNumber = "";
                                    JSONObject jsoncharges = response.getJSONObject("charges");
                                    if(response.has("amount_details") && response.getJSONObject("amount_details").has("tip")){
                                        JSONObject jsonObjectTip = response.getJSONObject("amount_details").getJSONObject("tip");
                                        if(jsonObjectTip.has("amount") && jsonObjectTip.getInt("amount")>0){
                                            DecimalFormat df = new DecimalFormat("#0.00");
                                            tipFromCardReader = df.format(jsonObjectTip.getInt("amount")/100.0);
                                        }
                                    }
                                    if (jsoncharges != null) {
                                        JSONArray data = jsoncharges.getJSONArray("data");
                                        if (data.length() == 0) return;
                                        JSONObject jsonFirstObject = data.getJSONObject(0);
                                        JSONObject payment_method_details = jsonFirstObject.getJSONObject("payment_method_details");
                                        if (payment_method_details != null) {
                                            JSONObject card_present = payment_method_details.getJSONObject("card_present");
                                            if (card_present != null && card_present.has("brand")) {
                                                cardName = card_present.getString("brand");
                                            }
                                            if (card_present != null && card_present.has("last4")) {
                                                cardNumber = card_present.getString("last4");
                                            }
                                        }
                                    }
                                } else {
                                    sendMessage("Something went wrong ", 2);
                                }
                            } catch (JSONException e) {
                                sendMessage("Something went wrong " + e.getMessage(), 2);
                                e.printStackTrace();
                            }
                            //Terminal.getInstance().disconnectReader(callback);
                            String finalTipFromCardReader = tipFromCardReader;
                            getActivity().runOnUiThread(() -> {
                                if (dialogDismissListener != null)
                                    dialogDismissListener.onDialogDismiss(paymentIntent, cardName, cardNumber, false, false, finalTipFromCardReader);
                                dismissAllowingStateLoss();
                            });
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            if (anError.getErrorCode() > 400) {
                                ApiError apiError = anError.getErrorAsObject(ApiError.class);
                                sendMessage("Payment Failed: " + apiError.getMessage(), 2);
                            } else {
                                sendMessage("Payment Failed: ", 2);
                            }
                            LogUtils.e("MOTO PAYMENT ERROR");
                            LogUtils.e(anError.getErrorBody());

                        }
                    });
        } catch (Exception e) {
            sendMessage("Something went wrong " + e.getMessage(), 2);
            e.printStackTrace();
        }
    }

    private void sendMessage(String message, int status) {
        Intent pushNotification = new Intent(Constants.CARD_READER_MESSAGE);
        pushNotification.putExtra("status", status);
        LogUtils.e("TERMINAL::" + message);
        if (message != null)
            pushNotification.putExtra("message", message);
        LocalBroadcastManager.getInstance(getActivity()).sendBroadcast(pushNotification);
    }

    BroadcastReceiver broadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            if (intent.getExtras() != null) {
                if (getActivity() != null) {
                    getActivity().runOnUiThread(() -> {
                        String message = intent.getStringExtra("message");
                        int status = intent.getIntExtra("status", 0);//0=neutral, 1=success, 2=failed

                        if (!Validators.isNullOrEmpty(message)) {
                            if (getActivity() != null)
                                getActivity().runOnUiThread(() -> {
                                    tvMessage.setText(intent.getStringExtra("message"));
                                    ClipboardManager clipboard = (ClipboardManager) getActivity().getSystemService(Context.CLIPBOARD_SERVICE);
                                    ClipData clip = ClipData.newPlainText("label", message);
                                    clipboard.setPrimaryClip(clip);
                                });
                        }
                        if (status == 2) {
                            pbLoading.setVisibility(View.GONE);
                            btnCancel.setVisibility(View.VISIBLE);
                            btnReStart.setVisibility(View.GONE);
                        } else if (status == RE_DISCOVER) {
                            pbLoading.setVisibility(View.GONE);
                            btnReStart.setVisibility(View.VISIBLE);
                            btnCancel.setVisibility(View.GONE);
                        } else {
                            btnCancel.setVisibility(View.GONE);
                            pbLoading.setVisibility(View.VISIBLE);
                            btnReStart.setVisibility(View.GONE);
                        }
                    });
                }

            }
        }
    };

    @Override
    public void onDismiss(@NonNull @NotNull DialogInterface dialog) {
        super.onDismiss(dialog);
        LogUtils.e("Dismissed");

        if (stripeReader != null) {
            try {
                //Terminal.getInstance().disconnectReader(callback);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        if (cancelableDiscovery != null) {
            cancelableDiscovery.cancel(callback);
        }
        if (dialogDismissListener != null) {
            dialogDismissListener.onDialogDismissPaymentFailed();
        }
        LocalBroadcastManager.getInstance(getActivity()).unregisterReceiver(broadcastReceiver);
    }

    //BluetoothReaderListener methods starts
    @Override
    public void onStartInstallingUpdate(@NotNull ReaderSoftwareUpdate readerSoftwareUpdate, @org.jetbrains.annotations.Nullable Cancelable cancelable) {
        sendMessage("Update installing", 0);
    }

    @Override
    public void onReportReaderSoftwareUpdateProgress(float v) {
        sendMessage("Updating " + MyApp.df.format(v * 100) + "%", 0);
    }

    @Override
    public void onFinishInstallingUpdate(@org.jetbrains.annotations.Nullable ReaderSoftwareUpdate readerSoftwareUpdate, @org.jetbrains.annotations.Nullable TerminalException e) {
        sendMessage("Finishing Updating", 0);
    }

    @Override
    public void onReportAvailableUpdate(@NotNull ReaderSoftwareUpdate readerSoftwareUpdate) {
        sendMessage("Update available", 0);
    }

    @Override
    public void onRequestReaderInput(@NotNull ReaderInputOptions readerInputOptions) {

    }

    @Override
    public void onRequestReaderDisplayMessage(@NotNull ReaderDisplayMessage readerDisplayMessage) {
        sendMessage("Message\n" + readerDisplayMessage.toString(), 0);
    }

    @Override
    public void onReportReaderEvent(@NotNull ReaderEvent readerEvent) {

    }

    @Override
    public void onReportLowBatteryWarning() {
        sendMessage("Low battery please recharge the reader", 0);
    }

    @Override
    public void onBatteryLevelUpdate(float v, @NonNull BatteryStatus batteryStatus, boolean b) {
        MyApp.getInstance().getMyPreferences().saveBatteryLevel(v);
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        // If we receive a response to our permission check, initialize
        if (requestCode == 100) {
            initialize();
        }
    }

    @Override
    public void onUnexpectedReaderDisconnect(@NonNull Reader reader) {
        if(!isTapToPayReaderDisconncetedMannuly){
            sendMessage("Unexpected Reader Disconnect " + reader.getSerialNumber() + ", Please Retry.", RE_DISCOVER);
        }
    }
    //BluetoothReaderListener methods end
}
